import 'dart:async';

import 'package:dropsource_app_utilities/dropsource_app_utilities.dart';
import 'package:firebase_crashlytics/firebase_crashlytics.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum AppEnvironment {
  development,
  staging,
  production,
  testing,
}

abstract class AppConfig<S> {
  AppConfig({
    @required this.appName,
    @required this.environment,
    @required this.apiBaseUrl,
    this.initializeCrashlytics = true,
    this.monitorConnectivity = true,
    this.enableCrashlyiticsInDevMode = true,
  });

  final String appName;
  final AppEnvironment environment;
  final String apiBaseUrl;
  final bool initializeCrashlytics,
      monitorConnectivity,
      enableCrashlyiticsInDevMode;

  Future startCrashlytics() async {
    if (this.initializeCrashlytics) {
      Crashlytics.instance.enableInDevMode = enableCrashlyiticsInDevMode;
      FlutterError.onError = (FlutterErrorDetails details) {
        Crashlytics.instance.onError(details);
      };
    }
  }

  Future<S> loadState();

  Widget createApp(S state);

  Future run() async {
    await startCrashlytics();
    final _state = await loadState();
    runApp(createApp(_state));
  }
}
